@@ -68,6 +68,7 @@ typedef struct pgspCtx { /* Used as `funcctx->user_fctx` in pg_show_plans(). */
 	pgspEntry       *pgsp_tmp_entry; /* PGSP entry currently processing. */
 	int              curr_nest; /* Current nest level porcessing. */
 	bool             is_done; /* Done processing current PGSP entry? */
+	int				 n_plans;
 } pgspCtx;
 
 /* Function Prototypes */
@@ -107,13 +108,7 @@ static void pgsp_shmem_request(void);
 #endif
 static void pgsp_shmem_startup(void);
 /* Saves query plans to the shared hash table. */
-static
-#if PG_VERSION_NUM < 180000
-void
-#else
-bool
-#endif
-pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags);
+static void pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags);
 /* Keeps track of the nest level. */
 static void pgsp_ExecutorRun(QueryDesc *queryDesc, ScanDirection direction,
                              uint64 count
@@ -469,31 +464,17 @@ pgsp_shmem_startup(void)
 	LWLockRelease(AddinShmemInitLock);
 }
 
-static
-#if PG_VERSION_NUM < 180000
-void
-#else
-bool
-#endif
+static void
 pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags)
 {
 	ExplainState *es;
-#if PG_VERSION_NUM >= 180000
-	bool ret_val;
-#endif
 
 	if (prev_ExecutorStart)
 	{
-#if PG_VERSION_NUM >= 180000
-		ret_val =
-#endif
 		prev_ExecutorStart(queryDesc, eflags);
 	}
 	else
 	{
-#if PG_VERSION_NUM >= 180000
-		ret_val =
-#endif
 		standard_ExecutorStart(queryDesc, eflags);
 	}
 
@@ -502,20 +483,12 @@ pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags)
 		        errcode(ERRCODE_OUT_OF_MEMORY),
 		        errmsg("not enough memory to append new query plans"),
 		        errhint("Try increasing 'pg_show_plans.max_plan_length'."));
-		return
-#if PG_VERSION_NUM >= 180000
-			ret_val
-#endif
-			;
+		return;
 	}
 
 	if (!pgsp->is_enabled)
 	{
-		return
-#if PG_VERSION_NUM >= 180000
-			ret_val
-#endif
-			;
+		return;
 	}
 
 	es = NewExplainState();
@@ -527,11 +500,7 @@ pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags)
 	append_query_plan(es);
 	pfree(es->str->data);
 
-	return
-#if PG_VERSION_NUM >= 180000
-		ret_val
-#endif
-		;
+	return;
 }
 
 static void
@@ -611,6 +580,7 @@ pg_show_plans(PG_FUNCTION_ARGS)
 		pgsp_ctx = (pgspCtx *)palloc(sizeof(pgspCtx));
 		pgsp_ctx->is_done = true;
 		pgsp_ctx->curr_nest = 0;
+		pgsp_ctx->n_plans = 0;
 		pgsp_ctx->hash_seq = (HASH_SEQ_STATUS *)palloc(sizeof(HASH_SEQ_STATUS));
 		hash_seq_init(pgsp_ctx->hash_seq, pgsp_hash);
 		funcctx->user_fctx = (void *)pgsp_ctx;
@@ -663,6 +633,8 @@ pg_show_plans(PG_FUNCTION_ARGS)
 				call_cntr++;
 			}
 			SpinLockAcquire(&pgsp_tmp_entry->mutex);
+			pgsp_ctx->n_plans = pgsp_tmp_entry->n_plans;
+			SpinLockRelease(&pgsp_tmp_entry->mutex);
 		}
 
 		/* A single hash entry may store multiple (nested) plans, so
@@ -679,7 +651,7 @@ pg_show_plans(PG_FUNCTION_ARGS)
 		values[4] = CStringGetTextDatum(pgsp_tmp_entry->plan + offset);
 		htup = heap_form_tuple(funcctx->tuple_desc, values, nulls);
 
-		if (curr_nest < pgsp_tmp_entry->n_plans-1)
+		if (curr_nest < pgsp_ctx->n_plans - 1)
 		{ /* Still have nested plans. */
 			curr_nest++;
 			call_cntr--; /* May not be legal, but it works. */
@@ -687,7 +659,7 @@ pg_show_plans(PG_FUNCTION_ARGS)
 		} else { /* No more nested plans, get a new entry. */
 			curr_nest = 0;
 			is_done = true;
-			SpinLockRelease(&pgsp_tmp_entry->mutex);
+			pgsp_ctx->n_plans = 0;
 		}
 		/* Save values back to the context. */
 		pgsp_ctx->is_done = is_done;
